// ../packages/core/index.ts
import * as process2 from "process";
import { log as log2 } from "../utils/index.js";
import { cac } from "cac";

// ../package.json
var version = "1.2.0";

// ../packages/core/storage.ts
import { existsSync, promises as fs, mkdirSync } from "fs";
import { resolve } from "path";
import { homedir } from "os";
var storage;
var storageDir = (() => {
  const home = homedir();
  const oldPath = resolve(home, "pr-checker-storage");
  if (existsSync(oldPath))
    return oldPath;
  const dotPath = resolve(home, ".config/pr-checker-storage");
  if (!existsSync(dotPath)) {
    mkdirSync(dotPath, {
      recursive: true
    });
  }
  return dotPath;
})();
var storagePath = resolve(storageDir, "_pr_checker_storage.json");
async function loadStorage(fn) {
  if (!storage) {
    storage = existsSync(storagePath) ? JSON.parse(await fs.readFile(storagePath, "utf-8") || "{}") || {} : {};
  }
  if (fn) {
    if (await fn(storage))
      await saveStorage();
  }
  return storage;
}
async function saveStorage() {
  if (storage) {
    if (!existsSync(storageDir))
      await fs.mkdir(storageDir, { recursive: true });
    await fs.writeFile(storagePath, JSON.stringify(storage), "utf-8");
  }
}
async function clearStorage() {
  if (storage) {
    if (!existsSync(storageDir))
      await fs.mkdir(storageDir, { recursive: true });
    await fs.writeFile(storagePath, "{}", "utf-8");
  }
}

// ../packages/core/runtime.ts
import prompts from "prompts";
import ora from "ora";
import { isEmptyObj, log } from "../utils/index.js";
import { Table } from "console-table-printer";
import chalk2 from "chalk";

// ../packages/core/command-option.ts
import chalk from "chalk";
var typeOption = [{
  type: "select",
  name: "typeSelect",
  message: "Detect all Repo's PR?",
  choices: ["All Repo", "Detect the PR of a certain Repo"].map((item) => {
    let value = "all";
    if (item !== "All Repo")
      value = "select";
    return { title: item, value };
  })
}];
var createRepoOption = (list) => {
  const handler = (item) => {
    return { title: item, value: item };
  };
  return [{
    type: "select",
    name: "RepoSelect",
    message: "Please select a Repo",
    choices: list.map(handler)
  }];
};
var createPrOption = (list) => {
  const handler = (item) => {
    const repo = chalk.blueBright.bold(`[${item.repo}]`);
    const number = chalk.yellowBright.bold(`[#${item.number}]`);
    const title = `${item.title}`;
    const canMerge = item.isNeedUpdate ? chalk.yellowBright.bold("<CanMerge:true>") : chalk.redBright.bold("<CanMerge:false>");
    const infoTitle = `${canMerge}: ${repo}-${number} -> ${title}`;
    return {
      title: infoTitle,
      value: {
        title: item.title,
        number: item.number,
        repo: item.repo,
        isNeedUpdate: item.isNeedUpdate,
        reason: item.reason,
        infoTitle
      }
    };
  };
  return [{
    type: "multiselect",
    name: "prSelect",
    message: "Please select the PR that needs to be updated",
    optionsPerPage: 20,
    choices: list.map(handler),
    onRender() {
      if (this.firstRender) {
        this.renderDoneOrInstructions = function() {
          return "";
        };
      }
    }
  }];
};

// ../packages/core/gitApi.ts
import { Octokit } from "@octokit/core";
var GitApi = class {
  octokit;
  owner;
  forkRepoCache;
  constructor(token, owner) {
    this.octokit = new Octokit({
      auth: token
    });
    this.owner = owner;
    this.forkRepoCache = {};
  }
  /**
   * 获取账户下所有 pr
   * @param username
   */
  async getPRList(username = this.owner) {
    try {
      const { data } = await this.octokit.request("GET /search/issues", {
        q: `is:pr is:open author:${username}`,
        per_page: 1e3
      });
      const res = {};
      data.items.forEach((val) => {
        const repo = val.repository_url.split("repos/")[1];
        if (!res[repo])
          res[repo] = [];
        res[repo].push({
          title: val.title,
          number: val.number,
          repo,
          id: val.id
        });
      });
      return res;
    } catch (error) {
      console.log(error);
      return [];
    }
  }
  /**
   * Get pr details according to pr number and repo name
   * @param pull_number pr 号
   * @param repo_name upstream Repo name
   * @param title
   */
  async getPRByRepo(pull_number, repo_name, title) {
    try {
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/pulls/{pull_number}`,
        {
          pull_number
        }
      );
      const forkRepoName = data.head.repo.full_name;
      const forkRepoDefaultBranch = data.head.repo.default_branch;
      if (!this.forkRepoCache[forkRepoName]) {
        const { data: forkRepoData } = await this.octokit.request(
          `GET /repos/${forkRepoName}/branches/{default_branch_name}`,
          {
            default_branch_name: forkRepoDefaultBranch
          }
        );
        this.forkRepoCache[forkRepoName] = {
          sha: forkRepoData.commit.sha
        };
      }
      return {
        sha: this.forkRepoCache[forkRepoName].sha,
        // Fork warehouse default branch hash
        org_repo: data.base.repo.full_name,
        org_repo_default_branch: data.base.repo.default_branch,
        pr_sha: data.head.sha,
        pr_repo: forkRepoName,
        pr_repo_default_branch: forkRepoDefaultBranch,
        merged: data.merged,
        mergeable: data.mergeable,
        mergeable_state: data.mergeable_state,
        number: pull_number,
        repo: repo_name,
        title: title || ""
      };
    } catch (error) {
      console.error(error);
      return {};
    }
  }
  /**
   * Determine whether pr wants to synchronize the upstream Repo
   * @param repo_name upstream Repo name
   * @param pr_info The getPRByRepo function returns the result
   */
  async needUpdate(repo_name, pr_info) {
    try {
      if (pr_info.mergeable_state === "dirty")
        return { isNeedUpdate: false, reason: "code conflict" };
      const { data } = await this.octokit.request(
        `GET /repos/${repo_name}/commits`
      );
      if (data.length >= 0) {
        const cur_sha = data[0].sha;
        if (pr_info.sha !== cur_sha && pr_info.mergeable && !pr_info.merged)
          return { isNeedUpdate: true };
        else
          return { isNeedUpdate: false, reason: "not updated" };
      } else {
        return { isNeedUpdate: false, reason: "unknown error" };
      }
    } catch (error) {
      console.error(error);
      return { isNeedUpdate: false, reason: "unknown error" };
    }
  }
  /**
   * update pr
   * @param pull_number
   * @param repo_name
   */
  async updatePR(pull_number, repo_name) {
    try {
      const res = await this.octokit.request(
        `PUT /repos/${repo_name}/pulls/{pull_number}/update-branch`,
        {
          pull_number
        }
      );
      return res.data.message;
    } catch (error) {
      console.error(error);
      return {};
    }
  }
};
async function getUserName(token) {
  const octokit = new Octokit({
    auth: token
  });
  const { data } = await octokit.request("GET /user");
  return data;
}

// ../packages/core/runtime.ts
var promptsRun = async (option) => {
  const res = await prompts(option, {
    onCancel: () => {
      log("info", "Operation cancel");
      process.exit();
    }
  });
  return { ...res };
};
async function runtimeStart(store) {
  const isAllRepo = await promptsRun(typeOption);
  let updateRes = [];
  const spinner = ora({ text: "Loading Repo......", color: "blue" }).start();
  const githubApi = new GitApi(store.token, store.username);
  const prList = await githubApi.getPRList();
  spinner.stop();
  const repoList = Object.keys(prList);
  if (isAllRepo.typeSelect === "select") {
    const selectRepo = await promptsRun(createRepoOption(repoList));
    const prl = prList[selectRepo.RepoSelect];
    log("info", `Checking PR by ${selectRepo.RepoSelect}......`);
    const prListByRepo = await checkPR(prl, githubApi);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    log("info", `Update PR by ${selectRepo.RepoSelect}......`);
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  } else {
    const prl = [];
    repoList.forEach((val) => {
      prList[val].forEach((item) => {
        prl.push(item);
      });
    });
    log("info", "Checking PR......");
    const prListByRepo = await checkPR(prl, githubApi);
    const prSelectRes = await promptsRun(createPrOption(prListByRepo));
    log("info", "Update PR......");
    updateRes = await updatePR(prl, prSelectRes, githubApi);
  }
  spinner.succeed();
  log("success", "\u2714 All PR updates completed");
  await printUpdateRes(updateRes);
}
async function checkPR(prl, githubApi) {
  if (prl.length === 0) {
    log("error", "Please select a pr to check");
    process.exit();
  }
  const prListByRepo = await Promise.all(createRunList(prl.length, async (i) => {
    const prInfo = await githubApi.getPRByRepo(prl[i].number, prl[i].repo, prl[i].title);
    const res = await githubApi.needUpdate(prl[i].repo, prInfo);
    log("success", `\u2714 Check PR #${prl[i].number} completed`);
    return {
      ...prInfo,
      ...res
    };
  }));
  return prListByRepo;
}
async function updatePR(prl, prSelectRes, githubApi) {
  const updateRes = await Promise.all(createRunList(prl.length, async (i) => {
    if (prSelectRes.prSelect[i] && prSelectRes.prSelect[i].isNeedUpdate) {
      await githubApi.updatePR(prSelectRes.prSelect[i].number, prSelectRes.prSelect[i].repo);
      log("success", `\u2714 Update PR #${prl[i].number} completed`);
    }
    return {
      ...prSelectRes.prSelect[i]
    };
  }));
  return updateRes;
}
async function printUpdateRes(res) {
  const p = new Table({
    columns: [
      { name: "number", alignment: "left" },
      { name: "can merge", alignment: "left" },
      { name: "success", alignment: "left" },
      { name: "reason", alignment: "left" },
      { name: "repo", alignment: "left" },
      { name: "title", alignment: "left" }
    ]
  });
  res.sort((a, b) => Number(a.isNeedUpdate) - Number(b.isNeedUpdate));
  res.forEach((item) => {
    if (!isEmptyObj(item)) {
      p.addRow({
        "number": chalk2.greenBright.bold(`#${item.number}`),
        "can merge": item.isNeedUpdate ? chalk2.greenBright.bold("true") : chalk2.redBright.bold("false"),
        "success": item.isNeedUpdate ? chalk2.greenBright.bold("true") : chalk2.redBright.bold("false"),
        "reason": item.isNeedUpdate ? chalk2.blueBright.bold("-") : item.reason === "not updated" ? chalk2.blueBright.bold(item.reason) : chalk2.yellowBright.bold(item.reason),
        "repo": chalk2.blueBright.bold(`<${item.repo}>`),
        "title": item.title
      });
    }
  });
  p.printTable();
}
function createRunList(taskNum, taskFunc) {
  const taskList = [];
  for (let i = 0; i < taskNum; i++) {
    taskList.push(new Promise((resolve2) => {
      resolve2(taskFunc(i));
    }));
  }
  return taskList;
}

// ../packages/core/index.ts
var cli = cac("pr-checker");
var run = async () => {
  const storage2 = await loadStorage();
  cli.option("-t <token>,--token <token>", "set github token");
  cli.option("-u <username>, --username <username>", "set github username");
  cli.option("-c, --clear", "clear token and username");
  cli.option("-g, --get", "get git config");
  cli.command("run", "check your pr").action(async () => {
    if (!storage2.token) {
      log2("error", "use `pr-checker -t <TOKEN>` to set your token");
      process2.exit(1);
    }
    if (!storage2.username) {
      log2("info", "You have not set a username, it has been automatically set for you according to the token");
      const { login } = await getUserName(storage2.token);
      await setUserName(login);
    }
    await runtimeStart(storage2);
  });
  cli.help();
  cli.version(version);
  const parseRes = cli.parse();
  const {
    u,
    username,
    t,
    token,
    c,
    clear,
    h,
    help,
    v,
    version: versions,
    g,
    get
  } = parseRes.options;
  if (h || help || v || versions)
    return;
  if (c || clear) {
    if (storage2) {
      await clearStorage();
      log2("success", "clear token and username successfully");
      return;
    }
  }
  if (g || get) {
    if (storage2) {
      log2("info", `username: ${storage2.username || "null"}`);
      log2("info", `token: ${storage2.token || "null"}`);
      return;
    }
  }
  if (t && typeof t === "string" || token && typeof token === "string") {
    storage2.token = t || token;
    await saveStorage();
    log2("success", "token set successfully");
    return;
  }
  if (u && typeof u === "string" || username && typeof username === "string") {
    await setUserName();
    log2("success", "username set successfully");
  }
  async function setUserName(name) {
    storage2.username = name || u || username;
    await saveStorage();
  }
};
export {
  run
};
